import log4js from 'log4js';
import path from 'path';

import config from '../config';

// 输出配置
const output = (ctx, message = '', commonInfo = {}) => {
  const {
    method, // 请求方式
    headers, // 请求headers
    origin, // 获取URL的来源，包括 protocol 和 host。
    originalUrl // 获取请求原始URL。
  } = ctx.request;
  interface client {
    originalUrl: string,
    userAgent: string,
    query?: string
  }
  const client: client = {
    originalUrl,
    userAgent: headers['user-agent']
  };
  if (ctx.querystring) {
    client.query = ctx.querystring;
  };
  let text = `[${method}] ${ctx.status} [${origin}${originalUrl}]
req: ${JSON.stringify(Object.assign(client, commonInfo))}`;

  if (Object.prototype.toString.call(message) === '[object Object]'
  || Object.prototype.toString.call(message) === '[object Error]') {
    message = JSON.stringify(message);
  };

  if (message) {
    text += '\n' + String(message);
  };
  return text;
};
// 日志级别
const methods = ['trace', 'debug', 'info', 'warn', 'error', 'fatal', 'mark'];

// 默认配置
const defaultConfig = {
  logLevel: 'debug', // 日志级别
  dir: path.resolve(__dirname, '../log'), // 指定日志存放目录名
  env: 'dev', // 指定当前环境，当开发时控制台也输出
  serverIp: '127.0.0.1'
};

// log配置
const opt = Object.assign({}, defaultConfig, config.logs);
const { env, appLogLevel, dir, serverIp } = opt;
/**
 * 记录日志的方式
 * 指定要记录的日志分类 log
 * 展示方式为文件类型 dateFile
 * 日志输出的文件名 s-yyyy-MM-dd.log
 */
interface appenders {
  [log: string]: {
    type: string,
    filename?: string,
    pattern?: string,
    alwaysIncludePattern?: boolean,
    serverIp?: string,
    layout: object
  }
}
const appenders: appenders = {
  log: {
    // 按日期进行输出log
    type: 'dateFile',
    // 路径和文件名
    filename: `${dir}/z`,
    pattern: 'yyyy-MM-dd.log',
    alwaysIncludePattern: true,
    serverIp,
    // 日志输出格式
    layout: {
      type: 'pattern',
      pattern: '[%d{yyyy-MM-dd hh:mm:ss}] [%p] %c %m%n'
    }
  }
};
// 开发和本地同时启用console
if (env === 'dev' || env === 'local' || env === 'development') {
  appenders.out = {
    type: 'console',
    layout: {
      type: 'pattern',
      pattern: '%[[%d{yyyy-MM-dd hh:mm:ss}] [%p] %c%] %m%n'
    }
  };
};
const isDev = env === 'dev' || env === 'local' || env === 'development';

/**
   * 指定日志的默认配置项
   * 如果 log4js.getLogger 中没有指定，默认为 appenders中的日志配置项, 数组
   * 指定日志的记录内容显示 某级别 及 某级别 以上级别的信息
   */
const categories = {
  default: {
    appenders: Object.keys(appenders),
    level: appLogLevel
  }
};

// log4js配置
log4js.configure({
  appenders,
  categories,
  pm2: !isDev,
  pm2InstanceVar: 'INSTANCE_ID'
});
// 初始化log4js
export const logger = log4js.getLogger('log');

export const middlewareLog = () => {
  // koa上下文存放的log输出方法
  const contextLogger = {};

  return async(ctx, next) => {
    methods.map(el => {
      contextLogger[el] = (message) => {
        logger[el](output(ctx, message, { serverIp }));
      };
    });
    ctx.$log = contextLogger;
    await next();
    if (ctx.response && ctx.status < 400) {
      interface commonInfo {
        serverIp: string,
        params?: any
      }
      const commonInfo: commonInfo = {
        serverIp,
      };
      if (ctx.request.body) {
        commonInfo.params = ctx.request.body;
      };
      // logger.info(output(ctx, null, commonInfo))
    }
    else {
      ctx.throw(ctx.status, ctx.response);
    }
  }
}
